/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.connector.impl.database;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

public class Database {

  private Connection connection;

  public Database(String driver) throws ClassNotFoundException, InstantiationException, IllegalAccessException {
    Class.forName(driver, true, Thread.currentThread().getContextClassLoader()).newInstance();
  }

  public void connect(String url, String username, String password) throws SQLException {
    connection = DriverManager.getConnection(url, username, password);
  }

  public void disconnect() throws SQLException {
    connection.close();
  }

  public ResultSet select(String query) throws SQLException {
    Statement statement = connection.createStatement(
        ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
    return statement.executeQuery(query);
  }

  public String[][] select2(String query) throws SQLException {
    Statement statement = connection.createStatement(
        ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
    ResultSet result = statement.executeQuery(query);
    ResultSetMetaData metadata = result.getMetaData();
    int columns = metadata.getColumnCount();
    int rows = 0;
    while (result.next()) {
      rows++;
    }
    result.first();
    String[][] results = new String[rows][columns];
    for (int i = 0; i < rows; i++) {
      for (int j = 0; j < columns; j++) {
        results[i][j] = result.getString(j + 1);
      }
      result.next();
    }
    return results;
  }
}
