/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.connector.impl.database;

public class H2Connector extends RemoteDatabaseConnector {

  private Boolean local = true;
  private Boolean server = false;
  private Boolean ssl = null;

  public Boolean isLocal() {
    return local;
  }

  public Boolean isServer() {
    return server;
  }

  public Boolean isSsl() {
    return ssl;
  }

  public void setLocal(Boolean local) {
    this.local = local;
  }

  public void setServer(Boolean server) {
    this.server = server;
  }

  public void setSsl(Boolean ssl) {
    this.ssl = ssl;
  }

  public void setHostName(String hostName) {
    super.setHostName(hostName);
  }

  public void setPort(Integer port) {
    super.setPort(port);
  }

  @Override
  protected String getDriver() {
    return "org.h2.Driver";
  }

  @Override
  protected String getUrl() {
    StringBuilder builder = new StringBuilder("jdbc:h2:");
    if (isServer() || isSsl()) {
      if (isServer()) {
        builder.append("tcp://");
      } else {
        builder.append("ssl://");
      }
      builder.append(getHostName());
      builder.append(":");
      builder.append(getPort());
      builder.append("/");
    } else if (isLocal()) {
      builder.append("file:");
    }
    builder.append(getDatabase());
    return builder.toString();
  }
}
