/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.connector.impl.database;

import java.util.ArrayList;
import java.util.List;

import org.ow2.bonita.connector.core.Connector;
import org.ow2.bonita.connector.core.ConnectorError;

public abstract class LocalDatabaseConnector extends Connector {

  private String database;
  private String query;
  private String username;
  private String password;
  private String[][] result = new String[0][0];

  public String getDatabase() {
    return database;
  }

  public String getQuery() {
    return query;
  }

  public String getUsername() {
    return username;
  }

  public String getPassword() {
    return password;
  }

  public String[][] getResults() {
    return result;
  }

  public String getResult() {
    StringBuilder builder = new StringBuilder();
    for (int i = 0; i < result.length; i++) {
      for (int j = 0; j < result[0].length; j++) {
        builder.append(result[i][j]);
        if (j + 1 < result[0].length) {
          builder.append(Connector.ATTRIBUTE_SEPARATOR);
        }
      }
      if (i + 1 < result.length) {
        builder.append(Connector.OBJECT_SEPARATOR);
      }
    }
    return builder.toString();
  }

  public void setDatabase(String database) {
    this.database = database;
  }

  public void setQuery(String query) {
    this.query = query;
  }

  public void setUsername(String username) {
    this.username = username;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public void setResult(String[][] result) {
    this.result = result;
  }

  @Override
  protected void executeConnector() throws Exception {
    Database database = new Database(this.getDriver());
    database.connect(this.getUrl(), username, password);
    result = database.select2(query);
    database.disconnect();
  }

  protected abstract String getUrl();

  protected abstract String getDriver();

  @Override
  protected List<ConnectorError> validateValues() {
    return new ArrayList<ConnectorError>();
  }
}
