/**
 * Copyright (C) 2009  BonitaSoft S.A..
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.connector.impl.mapper;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.ow2.bonita.connector.core.RoleResolver;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.GroovyUtil;

/**
 * 
 * @author Matthieu Chaffotte
 *
 */
public class UserListRoleResolver extends RoleResolver {

	private String users;

	public String getUsers() {
		return users;
	}

	public void setUsers(String users) {
		this.users = users;
	}

	@Override @SuppressWarnings("unchecked")
	protected void executeConnector() throws Exception {
		if (GroovyUtil.isGroovyExpression(getUsers())) {
			Object people = AccessorUtil.getRuntimeAPI().evaluateGroovyExpression(getUsers(), getInstanceUUID());
			if (people instanceof Collection<?>) {
				setMembers(new HashSet<String>((Collection<String>) people));
			} else if (people instanceof String) {
				setMembers((String) people);
			} else {
				throw new BonitaRuntimeException("The Groovy expression returns neither a Collection nor a String.");
			}
		} else {
			setMembers(getUsers());
		}
  }

	private void setMembers(String members) {
		String[] tempMembers = members.split(",");
		Set<String> membersSet = new HashSet<String>();
		for (int i = 0; i < tempMembers.length; i++) {
			membersSet.add(tempMembers[i].trim());
		}
		setMembers(membersSet);
	}
}
