/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.connector.impl.twitter;

import java.util.ArrayList;
import java.util.List;

import org.ow2.bonita.connector.core.Connector;
import org.ow2.bonita.connector.core.ConnectorError;

import twitter4j.Twitter;

public abstract class TwitterConnector extends Connector {

  private String proxyHost;
  private Integer proxyPort;
  private String proxyUser;
  private String proxyPass;
  private String senderId;
  private String senderPassword;

  public String getProxyHost() {
    return proxyHost;
  }

  public Integer getProxyPort() {
    return proxyPort;
  }

  public String getProxyUser() {
    return proxyUser;
  }

  public String getProxyPass() {
    return proxyPass;
  }

  public String getSenderId() {
    return senderId;
  }

  public String getSenderPassword() {
    return senderPassword;
  }

  public void setProxyHost(String proxyHost) {
    this.proxyHost = proxyHost;
  }

  public void setProxyPort(Integer proxyPort) {
    this.proxyPort = proxyPort;
  }

  public void setProxyPort(Long port) {
      setProxyPort(port.intValue());
  }

  public void setProxyUser(String proxyUser) {
    this.proxyUser = proxyUser;
  }

  public void setProxyPass(String proxyPass) {
    this.proxyPass = proxyPass;
  }

  public void setSenderId(String senderId) {
    this.senderId = senderId;
  }

  public void setSenderPassword(String senderPassword) {
    this.senderPassword = senderPassword;
  }

  @Override
  protected void executeConnector() throws Exception {
    Twitter twitter = new Twitter(senderId, senderPassword);
    if (getProxyHost() != null && getProxyPort() != null) {
      twitter.setHttpProxy(proxyHost, proxyPort);
      if (getProxyUser() != null && getProxyPass() != null) {
        twitter.setHttpProxyAuth(proxyUser, proxyPass);
      }
    }
    executeTask(twitter);
  }

  protected abstract void executeTask(Twitter twitter) throws Exception;

  @Override
  protected List<ConnectorError> validateValues() {
    List<ConnectorError> errors = new ArrayList<ConnectorError>();
    if (getProxyPort() != null) {
      if (this.getProxyPort() < 0) {
        errors.add(new ConnectorError("proxyPort",
            new IllegalArgumentException("cannot be less than 0!")));
      } else if (this.getProxyPort() > 65535) {
        errors.add(new ConnectorError("proxyPort",
            new IllegalArgumentException("cannot be greater than 65535!")));
      }
    }
    return errors;
  }
}
