package org.ow2.bonita.definition.activity;

import java.util.HashMap;
import java.util.Map;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.pvm.Execution;
import org.ow2.bonita.runtime.InternalExecution;
import org.ow2.bonita.runtime.InternalInstance;
import org.ow2.bonita.services.Recorder;
import org.ow2.bonita.services.info.ActivityInstanceCurrentInfo;
import org.ow2.bonita.services.info.ProcessInstanceCurrentInfo;
import org.ow2.bonita.services.util.ServiceEnvTool;


public class EndingNodeBehaviour extends AbstractActivity {

  /**
   * 
   */
  private static final long serialVersionUID = 5317011615783376030L;

  protected EndingNodeBehaviour() {}
  
  public EndingNodeBehaviour(final ActivityDefinition activityDef) {
    super(activityDef);
  }


  @Override
  protected boolean bodyStartAutomatically() {
    return true;
  }

  @Override
  protected boolean executeBusinessLogic(final Execution execution) {
    final InternalExecution internalExecution =  (InternalExecution) execution;
    final InternalInstance instance = internalExecution.getInstance();
    instance.setInstanceState(InstanceState.FINISHED);
    final Recorder recorder = ServiceEnvTool.getRecorder();
    final ActivityInstanceCurrentInfo activityInstanceCurrentInfo = new ActivityInstanceCurrentInfo(
        instance.getProcessDefinitionUUID(),
        instance.getUUID(),
        internalExecution.getCurrentActivityInstanceUUID(),
        this.activityDef.isAutomatic(),
        this.activityDef.getName(),
        this.activityDef.getLabel(),
        this.activityDef.getDescription(),
        internalExecution.getIterationId(),
        internalExecution.getActivityInstanceId(),
        internalExecution.getScopeVariables());

    recorder.recordBodyEnded(activityInstanceCurrentInfo);

    recorder.recordInstanceEnded(new ProcessInstanceCurrentInfo(
        instance.getProcessDefinitionUUID(),
        instance.getUUID(),
        (instance.getParentInstance() != null) ? instance.getParentInstance().getUUID() : null,
        internalExecution.getVariables(),
        ServiceEnvTool.getUserId())
    );

    final InternalExecution execToSignal = instance.getExecutionToSignal();
    if (execToSignal != null) {
      // We are in a subflow, create map of output parameters to signal parent instance
      Map<String, Object> parameterValues = null;
      Map<String, Object> instanceVariables = instance.getRootExecution().getVariables();
      if (instanceVariables != null) {
        parameterValues = new HashMap<String, Object>();
        parameterValues.putAll(instanceVariables);
      }
      
      execToSignal.signal(SubFlow.SUBFLOW_SIGNAL, parameterValues);
      instance.setExecutionToSignal(null);
    } else {
      // We are in main instance -> finish
      instance.finish();
    }
    return true;
  }

  @Override
  protected void end(InternalExecution internalExecution) {
    internalExecution = (InternalExecution) internalExecution.destroyScope(internalExecution.getNode());
    internalExecution.end();
    final InternalExecution parent = internalExecution.getParent();
    if (parent != null) {
      parent.removeExecution(internalExecution);
    }
  }
}
