package org.ow2.bonita.env.descriptor;

import org.hibernate.Session;
import org.ow2.bonita.env.EnvConstants;
import org.ow2.bonita.persistence.db.QuerierDbSessionImpl;
import org.ow2.bonita.pvm.internal.wire.WireContext;
import org.ow2.bonita.pvm.internal.wire.WireDefinition;
import org.ow2.bonita.pvm.internal.wire.WireException;
import org.ow2.bonita.pvm.internal.wire.descriptor.AbstractDescriptor;
import org.ow2.bonita.util.ExceptionManager;

/**
 *
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class QuerierDbSessionDescriptor extends AbstractDescriptor {

  private static final long serialVersionUID = 1L;

  protected String sessionName = null;

  public Object construct(WireContext wireContext) {

    // get the hibernate-session
    Session session = null;
    if (sessionName != null) {
      session = (Session) wireContext.get(sessionName);
      if (session == null) {
      	String message = ExceptionManager.getInstance().getFullMessage(
      			"bse_QDSD_1", sessionName, EnvConstants.QUERIER_DB_SESSION_TAG);
        throw new WireException(message);
      }
    } else {
      session = wireContext.get(Session.class);
      if (session == null) {
      	String message = ExceptionManager.getInstance().getFullMessage(
      			"bse_QDSD_2", EnvConstants.QUERIER_DB_SESSION_TAG);
        throw new WireException(message);
      }
    }
    return new QuerierDbSessionImpl(session);
  }

  public Class< ? > getType(WireDefinition wireDefinition) {
    return QuerierDbSessionImpl.class;
  }

  public void setSessionName(String sessionName) {
    this.sessionName = sessionName;
  }

  public void setName(String name) {
    this.name = name;
    super.setName(name);
  }
}
