/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Mofified by Charles Souillard - BonitaSoft S.A.
 **/
package org.ow2.bonita.persistence;

import java.util.Collection;
import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.pvm.session.DbSession;


public interface QuerierDbSession extends DbSession {

  /**
   * TASK INSTANCE
   */
  Set<ActivityFullInstance> getUserTasks(String userId, ActivityState taskState);
  Set<ActivityFullInstance> getUserInstanceTasks(String userId, 
      ProcessInstanceUUID instanceUUID, ActivityState taskState);
  ActivityFullInstance getTaskInstance(ActivityInstanceUUID taskUUID);
  Set<ActivityFullInstance> getTaskInstances(ProcessInstanceUUID instanceUUID);
  
  /**
   * PROCESS INSTANCE
   */
  ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID);
  Set<ProcessFullInstance> getProcessInstances();
  Set<ProcessFullInstance> getProcessInstances(ProcessDefinitionUUID processId);
  Set<ProcessFullInstance> getProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs);
  
  /**
   * ACTIVITY INSTANCE
   */
  ActivityFullInstance getActivityInstance(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId);
  Set<ActivityFullInstance> getActivityInstances(ProcessInstanceUUID instanceUUID);
  ActivityFullInstance getActivityInstance(ActivityInstanceUUID activityInstanceUUID);


  
  /**
   * PROCESS DEFINITION
   */
  Set<ProcessFullDefinition> getProcesses();
  Set<ProcessFullDefinition> getProcesses(String processId);
  Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState);
  Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState);
  ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID);
  ProcessFullDefinition getProcess(String processId, String version);
  ProcessFullDefinition getLastDeployedProcess(String processId, ProcessDefinition.ProcessState processState);
  Set<ProcessFullDefinition> getDependentProcesses(ProcessDefinitionUUID processUUID);
  
  
  /**
   * ACTIVITY DEFINITION
   */
  ActivityDefinition getActivityDefinition(ActivityDefinitionUUID activityDefinitionUUID);
  Set<ProcessFullInstance> getProcessInstancesWithTaskState(Collection<ActivityState> activityStates);

}


