/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.persistence;

import java.util.HashSet;
import java.util.Set;

import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.TaskInstance;


public final class QuerierUtil {

  private QuerierUtil() { }

  public static Set<ActivityFullInstance> getUserTasks(Set<ActivityFullInstance> activities,
      String userId, ActivityState taskState) {
    Set<ActivityFullInstance> result = new HashSet<ActivityFullInstance>();
    if (activities != null) {
      for (ActivityFullInstance activity : activities) {
        if (activity.isTask() && (activity.getTask()).getState().equals(taskState)) {
          TaskInstance taskInstance = activity.getTask();
          String taskUserId = null;
          boolean userIdOk = false;
          boolean candidatesOk = false;
          try {
            taskUserId = taskInstance.getTaskUser();
            userIdOk = taskUserId != null && taskUserId.equals(userId);
          } catch (IllegalStateException e) {
            // nothing
          }
          if (!userIdOk) {
            try {
              Set<String> taskCandidates = taskInstance.getTaskCandidates();
              candidatesOk = taskCandidates != null && taskCandidates.contains(userId);
            } catch (IllegalStateException e) {
              // nothing
            }
          }
          if (userIdOk || (taskUserId == null && candidatesOk)) {
            result.add((ActivityFullInstance) taskInstance);
          }
        }
      }
    }
    return result;
  }
}


