/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.persistence.db;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.ow2.bonita.definition.ClassData;
import org.ow2.bonita.definition.InternalProcess;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.runtime.InternalInstance;


public class DbRepositoryBuffer {

  private final Map<ProcessInstanceUUID, InternalInstance> instances = new HashMap<ProcessInstanceUUID, InternalInstance>();
  private final Map<ProcessInstanceUUID, InternalInstance> removedInstances = new HashMap<ProcessInstanceUUID, InternalInstance>();

  private final Map<ProcessDefinitionUUID, InternalProcess> processes = new HashMap<ProcessDefinitionUUID, InternalProcess>();
  private final Map<ProcessDefinitionUUID, InternalProcess> removedProcesses = new HashMap<ProcessDefinitionUUID, InternalProcess>();
  
  private final Map<ProcessDefinitionUUID, Collection<ClassData>> processClassDatas =
    new HashMap<ProcessDefinitionUUID, Collection<ClassData>>();
  private final Map<ProcessDefinitionUUID, Collection<ClassData>> removedProcessClassDatas =
    new HashMap<ProcessDefinitionUUID, Collection<ClassData>>();

  private final Map<String, ClassData> globalClassDatas =
    new HashMap<String, ClassData>();
  private final Map<String, ClassData> removedGlobalClassDatas =
    new HashMap<String, ClassData>();

  public DbRepositoryBuffer() { }

  //All methods returning a Collection must never return null!
  //In that case, an empty collection must be returned


  /**
   * PROCESSES
   */
  public Map<ProcessDefinitionUUID, InternalProcess> getProcesss() {
    return this.processes;
  }
  public InternalProcess getProcess(final ProcessDefinitionUUID processUUID) {
    return this.processes.get(processUUID);
  }

  public void addProcess(final InternalProcess process) {
    final ProcessDefinitionUUID processUUID = process.getUUID();
    this.processes.put(processUUID, process);
    this.removedProcesses.remove(processUUID);
  }
  public void removeProcess(final InternalProcess process) {
    final ProcessDefinitionUUID processUUID = process.getUUID();
    this.removedProcesses.put(processUUID, process);
    this.processes.remove(processUUID);
  }
  public boolean isProcessRemoved(final ProcessDefinitionUUID processUUID) {
    return this.removedProcesses.containsKey(processUUID);
  }
  public boolean containsProcess(final ProcessDefinitionUUID processUUID) {
    return this.processes.containsKey(processUUID);
  }

  /**
   * INSTANCES
   */
  public Map<ProcessInstanceUUID, InternalInstance> getInstances() {
    return this.instances;
  }
  public InternalInstance getInstance(final ProcessInstanceUUID instanceUUID) {
    return this.instances.get(instanceUUID);
  }

  public void addInstance(final InternalInstance instance) {
    final ProcessInstanceUUID instanceUUID = instance.getUUID();
    this.instances.put(instanceUUID, instance);
    this.removedInstances.remove(instanceUUID);
  }
  public void removeInstance(final InternalInstance instance) {
    final ProcessInstanceUUID instanceUUID = instance.getUUID();
    this.removedInstances.put(instanceUUID, instance);
    this.instances.remove(instanceUUID);
  }
  public boolean isInstanceRemoved(final ProcessInstanceUUID instanceUUID) {
    return this.removedInstances.containsKey(instanceUUID);
  }
  public boolean containsInstance(final ProcessInstanceUUID instanceUUID) {
    return this.instances.containsKey(instanceUUID);
  }
  
  /**
   * PROCESS CLASS DATA
   */
  public Map<ProcessDefinitionUUID, Collection<ClassData>> getProcessClassDatas() {
    return this.processClassDatas;
  }
  public Collection<ClassData> getProcessClassData(final ProcessDefinitionUUID processUUID) {
    return this.processClassDatas.get(processUUID);
  }
  public void addProcessClassDatas(final ProcessDefinitionUUID processUUID, final Collection<ClassData> processCD) {
    this.processClassDatas.put(processUUID, processCD);
    this.removedProcessClassDatas.remove(processUUID);
  }
  public void removeProcessClassDatas(final ProcessDefinitionUUID processUUID) {
    this.removedProcessClassDatas.put(processUUID, this.processClassDatas.remove(processUUID));
  }
  public boolean isProcessClassDataRemoved(final ProcessDefinitionUUID processUUID) {
    return this.removedProcessClassDatas.containsKey(processUUID);
  }
  public boolean containsProcessClassData(final ProcessDefinitionUUID processUUID) {
    return this.processClassDatas.containsKey(processUUID);
  }

  /**
   * GLOBAL CLASS DATA
   */
  public Map<String, ClassData> getGlobalClassDatas() {
    return this.globalClassDatas;
  }
  public ClassData getGlobalClassData(final String className) {
    return this.globalClassDatas.get(className);
  }
  public void addGlobalClassData(final ClassData globalCD) {
    this.globalClassDatas.put(globalCD.getClassName(), globalCD);
    this.removedGlobalClassDatas.remove(globalCD.getClassName());
  }
  public void removeGlobalClassData(final ClassData globalCD) {
    final String className = globalCD.getClassName();
    this.removedGlobalClassDatas.put(className, globalCD);
    this.globalClassDatas.remove(className);
  }
  public boolean isGlobalClassDataRemoved(final String className) {
    return this.removedGlobalClassDatas.containsKey(className);
  }
  public boolean containsGlobalClassData(final String className) {
    return this.globalClassDatas.containsKey(className);
  }


}


