/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.internal.cmd;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.pvm.model.OpenProcessDefinition;
import org.ow2.bonita.pvm.session.PvmDbSession;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.Command;
import org.ow2.bonita.util.ExceptionManager;

/**
 * @author Tom Baeyens
 */
public class FindProcessDefinitionCmd implements Command<OpenProcessDefinition> {

  private static final long serialVersionUID = 1L;

  protected String processDefinitionName;
  protected int processDefinitionVersion;

  public FindProcessDefinitionCmd(String processDefinitionName,
      int processDefinitionVersion) {
    if (processDefinitionName == null) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_FPDC_1");
      throw new BonitaRuntimeException(message);
    }
    this.processDefinitionName = processDefinitionName;
    this.processDefinitionVersion = processDefinitionVersion;
  }

  public OpenProcessDefinition execute(Environment environment)
      throws Exception {
    PvmDbSession pvmDbSession = environment.get(PvmDbSession.class);
    return pvmDbSession.findProcessDefinitionByName(processDefinitionName,
        processDefinitionVersion);
  }
}
