/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.internal.deploy;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.pvm.Deployment;
import org.ow2.bonita.pvm.client.ClientProcessDefinition;
import org.ow2.bonita.pvm.internal.model.PVMProcessDefinitionImpl;
import org.ow2.bonita.pvm.session.PvmDbSession;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.ExceptionManager;
import org.ow2.bonita.util.Log;

/**
 * @author Tom Baeyens
 */
public class CheckVersion implements Deployer {

  private static final Log log = Log.getLog(CheckVersion.class.getName());

  protected boolean assign = true;

  public void deploy(Deployment deployment) {
    PVMProcessDefinitionImpl processDefinition = (PVMProcessDefinitionImpl) deployment
        .getProcessDefinition();

    String name = processDefinition.getName();
    if (name == null) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_CV_1");
      throw new BonitaRuntimeException(message);
    }

    int version = processDefinition.getVersion();
    if ((version == PVMProcessDefinitionImpl.UNASSIGNED_VERSION) && !assign) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_CV_2", name);
      throw new BonitaRuntimeException(message);
    }

    Environment environment = Environment.getCurrent();
    if (environment == null) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_CV_3");
      throw new BonitaRuntimeException(message);
    }

    PvmDbSession pvmDbSession = environment.get(PvmDbSession.class);
    if (pvmDbSession == null) {
    	String message = ExceptionManager.getInstance().getFullMessage(
    			"bp_CV_4", PvmDbSession.class.getName(), getClass().getName());
      throw new BonitaRuntimeException(message);
    }

    if ((version == PVMProcessDefinitionImpl.UNASSIGNED_VERSION) && assign) {
      ClientProcessDefinition latestDeployedVersion = pvmDbSession
          .findLatestProcessDefinitionByName(name);
      if (latestDeployedVersion != null) {
        version = latestDeployedVersion.getVersion() + 1;
      } else {
        version = 1;
      }
      log.debug("assigning version " + version + " to process definition "
          + name);
      processDefinition.setVersion(version);

    } else {
      if (pvmDbSession.findProcessDefinitionByName(name, version) != null) {
      	String message = ExceptionManager.getInstance().getFullMessage(
      			"bp_CV_5", name, version);
        throw new BonitaRuntimeException(message);
      }
    }
  }
}
