/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.internal.deploy;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.pvm.Deployment;
import org.ow2.bonita.pvm.ProcessDefinition;
import org.ow2.bonita.pvm.session.PvmDbSession;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.ExceptionManager;

/**
 * @author Tom Baeyens
 */
public class SaveProcess implements Deployer {

  public void deploy(Deployment deployment) {
    Environment environment = Environment.getCurrent();
    if (environment == null) {
    	String message = ExceptionManager.getInstance().getFullMessage(
    			"bp_SP_1", getClass().getName());
      throw new BonitaRuntimeException(message);
    }

    PvmDbSession pvmDbSession = environment.get(PvmDbSession.class);
    if (pvmDbSession == null) {
    	String message = ExceptionManager.getInstance().getFullMessage(
    			"bp_SP_2", PvmDbSession.class.getName(), getClass().getName());
      throw new BonitaRuntimeException(message);
    }

    ProcessDefinition processDefinition = deployment.getProcessDefinition();
    if (processDefinition == null) {
    	String message = ExceptionManager.getInstance().getFullMessage(
    			"bp_SP_3", getClass().getName());
      throw new BonitaRuntimeException(message);
    }

    pvmDbSession.save(processDefinition);
  }

}
