/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.internal.ejb;

import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.ow2.bonita.pvm.internal.cmd.CommandService;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.Command;
import org.ow2.bonita.util.ExceptionManager;

/**
 * Local, stateless session bean implementation of the command service.
 * 
 * @author Tom Baeyens
 */
public class EnterpriseLocalCommandService implements CommandService {

  private LocalCommandExecutor commandExecutor;

  public LocalCommandExecutor getCommandExecutor() {
    if (commandExecutor == null) {
      try {
        Context context = new InitialContext();
        LocalCommandExecutorHome commandExecutorHome = (LocalCommandExecutorHome) context
            .lookup("java:comp/env/ejb/LocalCommandExecutor");
        context.close();

        commandExecutor = commandExecutorHome.create();
      } catch (NamingException e) {
      	String message = ExceptionManager.getInstance().getFullMessage("bp_ELCS_1");
        throw new BonitaRuntimeException(message, e);
      } catch (CreateException e) {
      	String message = ExceptionManager.getInstance().getFullMessage("bp_ELCS_2");
        throw new BonitaRuntimeException(message, e);
      }
    }
    return commandExecutor;
  }

  public <T> T execute(Command<T> command) {
    return getCommandExecutor().execute(command);
  }

}
