package org.ow2.bonita.pvm.internal.jobexecutor;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

import org.ow2.bonita.util.Log;


public class DynamicJobExecutorThreadPool implements JobExecutorThreadPool {

  private ThreadFactory threadFactory;
  private ExecutorService executorService;
  private JobExecutor jobExecutor;
  @SuppressWarnings("unused")
  private static final Log log = Log.getLog(DynamicJobExecutorThreadPool.class.getName());
  
  public DynamicJobExecutorThreadPool(JobExecutor jobExecutor) {
    this.jobExecutor = jobExecutor;
  }
  
  public void start() {
    threadFactory = new JobExecutorThreadFactory(jobExecutor);
    executorService = Executors.newCachedThreadPool(threadFactory);
  }

  public void stop() {
    stop(false);
  }

  public void stop(boolean join) {
    /*for (JobExecutorThread jobExecutorThread : executorService.invokeAll(tasks)) {
      jobExecutorThread.stop(join);
    }*/
    executorService.shutdown();
  }

  private static class JobExecutorThreadFactory implements ThreadFactory {
    private JobExecutor jobExecutor;
    private int threadNb = 0;
    private static final Log log = Log.getLog(JobExecutorThreadFactory.class.getName());
    public JobExecutorThreadFactory(JobExecutor jobExecutor) {
      this.jobExecutor = jobExecutor;
    }
    public Thread newThread(Runnable r) {
      log.debug("Creating a new JobExecutorThread from " + this.getClass());
      return new JobExecutorThread(getNextThreadName(), jobExecutor);
    }
    protected String getNextThreadName() {
      return "JobExecutorThread" + getNextThreadNb();
    }
    private synchronized int getNextThreadNb() {
      return threadNb++; 
    }
  }
}
