/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.internal.model.op;

import java.util.Map;

import org.ow2.bonita.pvm.activity.ExternalActivity;
import org.ow2.bonita.pvm.internal.job.MessageImpl;
import org.ow2.bonita.pvm.internal.model.ExecutionImpl;
import org.ow2.bonita.pvm.internal.model.NodeImpl;
import org.ow2.bonita.pvm.internal.model.ExecutionImpl.Propagation;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.ExceptionManager;
import org.ow2.bonita.util.Log;

/**
 * @author Tom Baeyens
 */
public class Signal implements AtomicOperation {

  private static final Log log = Log.getLog(Signal.class.getName());

  String signalName;
  Map<String, Object> parameters;
  NodeImpl node;

  public Signal(String signalName, Map<String, Object> parameters, NodeImpl node) {
    this.signalName = signalName;
    this.parameters = parameters;
    this.node = node;
  }

  public boolean isAsync(ExecutionImpl execution) {
    return execution.getNode().isSignalAsync();
  }

  public void perform(ExecutionImpl execution) {
    if (execution.getName() != null) {
      log.debug(execution.toString() + " signals " + node);
    } else {
      log.debug("signalling " + node + ", signal=" + signalName);
    }

    ExternalActivity externalActivity = (ExternalActivity) node.getBehaviour();

    try {
      execution.setPropagation(Propagation.UNSPECIFIED);
      externalActivity.signal(execution, signalName, parameters);

    } catch (RuntimeException e) {
      throw e;

    } catch (Exception e) {
    	String message = ExceptionManager.getInstance().getFullMessage(
    			"bp_S_1", node, e.getMessage());
      throw new BonitaRuntimeException(message,e);
    }

    if (execution.getPropagation() == Propagation.UNSPECIFIED) {
      execution.proceed();
    }
  }

  public String toString() {
    return "signal(node)";
  }

  public MessageImpl<?> createAsyncMessage(ExecutionImpl execution) {
    return new SignalMessage(execution, signalName, node);
  }
}
