/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.internal.model.op;

import org.ow2.bonita.env.Environment;
import org.ow2.bonita.pvm.internal.job.MessageImpl;
import org.ow2.bonita.pvm.internal.jobexecutor.JobDbSession;
import org.ow2.bonita.pvm.internal.model.ExecutionImpl;
import org.ow2.bonita.pvm.internal.model.NodeImpl;

/**
 * @author Tom Baeyens
 */
public class SignalMessage extends MessageImpl<Object> {

  private static final long serialVersionUID = 1L;

  String signalName;
  NodeImpl node;

  public SignalMessage() {
  }

  public SignalMessage(ExecutionImpl execution, String signalName, NodeImpl node) {
    super(execution);
    this.signalName = signalName;
    this.node = node;
  }

  public Object execute(Environment environment) throws Exception {
    unlockExecution();

    Signal signal = new Signal(signalName, null, node);
    execution.performAtomicOperationSync(signal);

    JobDbSession jobDbSession = environment.get(JobDbSession.class);
    jobDbSession.delete(this);

    return null;
  }

  public String toString() {
    return "signal-transition-message[" + dbid + "]";
  }
}
