package org.ow2.bonita.pvm.internal.util;

import java.util.ArrayList;
import java.util.List;

import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.ExceptionManager;

/**
 * default implementation of the {@link Observable} interface.
 * 
 * @author Tom Baeyens
 */
public class DefaultObservable implements Observable {

  protected List<Listener> listeners = null;

  public void addListener(Listener listener) {
    if (listener == null) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_DO_1");
      throw new BonitaRuntimeException(message);
    }
    if (listeners == null) {
      listeners = new ArrayList<Listener>();
    }
    listeners.add(listener);
  }

  public void removeListener(Listener listener) {
    if (listener == null) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_DO_2");
      throw new BonitaRuntimeException(message);
    }
    if (listeners != null) {
      listeners.remove(listener);
    }
  }

  public Listener addListener(Listener listener, String eventName) {
    if (eventName == null) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_DO_3");
      throw new BonitaRuntimeException(message);
    }

    List<String> eventNames = new ArrayList<String>();
    eventNames.add(eventName);

    return addListener(listener, eventNames);
  }

  public Listener addListener(Listener listener, List<String> eventNames) {
    if (listener == null) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_DO_4");
      throw new BonitaRuntimeException(message);
    }
    if (eventNames == null) {
    	String message = ExceptionManager.getInstance().getFullMessage("bp_DO_5");
      throw new BonitaRuntimeException(message);
    }
    FilterListener filterListener = new FilterListener(listener, eventNames);
    addListener(filterListener);
    return filterListener;
  }

  public void fire(String eventName) {
    fire(eventName, null);
  }

  public void fire(String eventName, Object info) {
    if (listeners != null) {
      for (Listener listener : listeners) {
        listener.event(this, eventName, info);
      }
    }
  }

  public List<Listener> getListeners() {
    return listeners;
  }
}
