package org.ow2.bonita.pvm.internal.wire.descriptor;

import org.ow2.bonita.pvm.internal.wire.WireContext;
import org.ow2.bonita.pvm.internal.wire.WireException;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.ExceptionManager;
import org.ow2.bonita.util.ReflectUtil;

/**
 * loads the class with the specified class name using the WireContext class
 * loader.
 * 
 * @see WireContext#getClassLoader()
 * 
 * @author Tom Baeyens
 * @author Guillaume Porcher (documentation)
 */
public class ClassDescriptor extends AbstractDescriptor {

  private static final long serialVersionUID = 1L;

  String text;

  /**
   * loads the class from the class loader of the specified WireContext.
   * 
   * @throws WireException
   *           if the class could not be loaded.
   */
  public Object construct(WireContext wireContext) {
    ClassLoader classLoader = wireContext.getClassLoader();
    try {
      return ReflectUtil.loadClass(classLoader, text);
    } catch (BonitaRuntimeException e) {
      Throwable cause = e.getCause();
      String message = ExceptionManager.getInstance().getFullMessage(
      		"bp_CD_1", text, cause.getMessage());
      throw new WireException(message, cause);
    }
  }

  public void setClassName(String className) {
    this.text = className;
  }

  public void setClass(Class<?> clazz) {
    if (clazz == null) {
      text = null;
    } else {
      this.text = clazz.getName();
    }
  }
}
