/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.bonita.pvm.internal.wire.descriptor;

import org.hibernate.Session;
import org.ow2.bonita.pvm.internal.hibernate.HibernateJobDbSession;
import org.ow2.bonita.pvm.internal.wire.WireContext;
import org.ow2.bonita.pvm.internal.wire.WireDefinition;
import org.ow2.bonita.pvm.internal.wire.WireException;
import org.ow2.bonita.util.ExceptionManager;

/**
 * @author Tom Baeyens
 */
public class HibernateJobDbSessionDescriptor extends AbstractDescriptor {

  private static final long serialVersionUID = 1L;

  String sessionName;

  public Object construct(WireContext wireContext) {
    return new HibernateJobDbSession();
  }

  public void initialize(Object object, WireContext wireContext) {
    // get the hibernate-session
    Session session = null;
    if (sessionName != null) {
      session = (Session) wireContext.get(sessionName);
    } else {
      session = wireContext.get(Session.class);
    }

    if (session == null) {
    	String message = ExceptionManager.getInstance().getFullMessage(
    			"bp_HJDS_1", (sessionName != null ? "'" + sessionName + "'" : "by type "));
      throw new WireException(message);
    }

    // inject the session
    ((HibernateJobDbSession) object).setSession(session);
  }

  public Class<?> getType(WireDefinition wireDefinition) {
    return HibernateJobDbSession.class;
  }

  public void setSessionName(String sessionName) {
    this.sessionName = sessionName;
  }
}
