/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.runtime;

import java.util.Stack;


public class ProcessVersion {

  private String processId;
  private String processVersion;
  private Stack<String> previousVersions;

  @SuppressWarnings("unused")
  private ProcessVersion() { }
   
  public ProcessVersion(String processId, String version) {
    super();
    this.processId = processId;
    this.processVersion = version;
  }

  public String getProcessId() {
    return processId;
  }

  
  public String getProcessVersion() {
    return processVersion;
  }
  
  public void setProcessVersion(String version) {
    if (previousVersions == null)  {
      previousVersions = new Stack<String>();
    }
    if (this.processVersion != null) {
      previousVersions.add(this.processVersion);
    }
    this.processVersion = version;
  }
  
  //returns false if no version exists anymore
  public boolean removeVersion(String version) {
    //2 cases
    //1) the given version is the last one
    //2) the given version is a previous one
    if (this.processVersion != null && this.processVersion.equals(version)) {
      this.processVersion = null;
      if (previousVersions != null && !previousVersions.isEmpty()) {
        this.processVersion = previousVersions.pop();
      }  
    } else if (this.previousVersions != null) {
      this.previousVersions.remove(version);
    }
    
    return processVersion == null && (previousVersions == null || previousVersions.isEmpty()); 
  }
}


