/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Mofified by Charles Souillard - BonitaSoft S.A.
 **/
package org.ow2.bonita.services;

import java.util.Collection;
import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;

/**
 * @author Pierre Vigneras, Charles Souillard
 */
public interface Querier {

  String DEFAULT_KEY = "queryList";

  /*
   * ACTIVITY
   */
  Set<ActivityFullInstance> getActivityInstances(ProcessInstanceUUID instanceUUID);
  ActivityFullInstance getActivityInstance(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId);
  ActivityFullInstance getActivityInstance(ActivityInstanceUUID activityInstanceUUID);

  /*
   * HOOK
   */
  //Map<JavaHook.Type, HookInstance> getHooks(String processUUID, String activityId);

  /*
   * INSTANCE
   */
  Set<ProcessFullInstance> getProcessInstances();
  Set<ProcessFullInstance> getProcessInstances(ProcessDefinitionUUID processUUID);
  Set<ProcessFullInstance> getProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs);
  ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID);
  Set<ProcessFullInstance> getProcessInstancesWithTaskState(Collection<ActivityState> acitivtyStates);

  /*
   * TASK
   */
  Set<ActivityFullInstance> getTaskInstances(ProcessInstanceUUID instanceUUID);
  ActivityFullInstance getTaskInstance(ActivityInstanceUUID taskUUID);
  /** return all tasks associated to the given user and having the given state */
  Set<ActivityFullInstance> getUserTasks(String userId, ActivityState taskState);
  /** return all tasks associated to the given user and the given instance and having the given state */
  Set<ActivityFullInstance> getUserInstanceTasks(String userId, 
      ProcessInstanceUUID instanceUUID, ActivityState taskState);
  
  /*
   * DEFINITION
   */

  Set<ProcessFullDefinition> getProcesses();
  Set<ProcessFullDefinition> getProcesses(String processId);
  Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState);
  Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState);
  ProcessFullDefinition getProcess(String processId, String version);
  ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID);
  ProcessFullDefinition getLastDeployedProcess(String processId, ProcessDefinition.ProcessState processState);

  Set<ProcessFullDefinition> getDependentProcesses(ProcessDefinitionUUID processUUID);

  ActivityDefinition getActivity(ActivityDefinitionUUID activityDefinitionUUID);

}
