/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services;

import org.ow2.bonita.env.binding.JournalBinding;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.services.info.ActivityInstanceCurrentInfo;
import org.ow2.bonita.services.info.ProcessInstanceCurrentInfo;
import org.ow2.bonita.services.info.TaskCurrentInfo;

/**
 * A Recorder is responsible for storing <strong>runtime</strong> instances
 * such as currently deployed processes, running instances,
 * activities and tasks.
 *
 * Usually, recorded records are available using queries on a {@link JournalBinding}.
 *
 * @author Pierre Vigneras
 * @see JournalBinding
 */
public interface Recorder {
  String DEFAULT_KEY = "recorder";

  /*
   * ACTIVITY
   */
  void recordEnterActivity(ActivityInstanceCurrentInfo activityInstanceCurrentInfo);
  void recordBodyStarted(ActivityInstanceCurrentInfo activityInstanceCurrentInfo);
  void recordBodyEnded(ActivityInstanceCurrentInfo activityInstanceCurrentInfo);
  void recordBodyAborted(ActivityInstanceCurrentInfo activityInstanceCurrentInfo);
  void recordBodyCancelled(ActivityInstanceCurrentInfo activityInstanceCurrentInfo);


  /*
   * INSTANCE
   */
  void recordInstanceStarted(ProcessInstanceCurrentInfo processInstanceCurrentInfo);
  void recordInstanceEnded(ProcessInstanceCurrentInfo processInstanceCurrentInfo);
  void recordInstanceAborted(ProcessInstanceCurrentInfo processInstanceCurrentInfo);
  void recordInstanceCancelled(ProcessInstanceCurrentInfo processInstanceCurrentInfo);

  /*
   * TASK
   */
  void recordTaskReady(TaskCurrentInfo taskCurrentInfo);
  void recordTaskStarted(TaskCurrentInfo taskCurrentInfo);
  void recordTaskFinished(TaskCurrentInfo taskCurrentInfo);
  void recordTaskSuspended(TaskCurrentInfo taskCurrentInfo);
  void recordTaskResumed(TaskCurrentInfo taskCurrentInfo);
  void recordTaskAssigned(TaskCurrentInfo taskCurrentInfo);
  /*
   * DEFINITION
   */
  void recordProcessDeployed(ProcessFullDefinition processDef);
  void recordProcessUndeployed(ProcessFullDefinition processDef, String userId);

  /*
   * VARIABLE UPDATES
   */

  void recordInstanceVariableUpdated(String variableId, Object variableValue, ProcessInstanceUUID instanceUUID, String userId);
  void recordActivityVariableUpdated(String variableId, Object variableValue, ActivityInstanceUUID activityUUID, String userId);

  /**
   * Generic method for removing a journal record.
   *
   * Implementation may delegate to overloaded methods according to record type.
   * @param processInst the record to archive
   */
  void remove(ProcessFullInstance processInst);
  void remove(ProcessFullDefinition processDef);

}
