package org.ow2.bonita.services;

import java.util.Collection;
import java.util.Set;

import org.ow2.bonita.definition.ClassData;
import org.ow2.bonita.definition.InternalProcess;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.pvm.job.Timer;
import org.ow2.bonita.runtime.InternalExecution;
import org.ow2.bonita.runtime.InternalInstance;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public interface Repository {

  /**
   * The key used by the IoC engine for concrete implementation injection.
   */
  String DEFAULT_KEY = "repository";

  /**
   * CLASS
   */
  void storeProcessClassDatas(ProcessDefinitionUUID processUUID, Collection<ClassData> processClassDatas);
  void removeProcessClassDatas(ProcessDefinitionUUID processUUID);
  //if found in process datas take it, else look into global datas
  Collection<ClassData> getProcessClassDatas(ProcessDefinitionUUID processUUID);
  void storeGlobalClassData(ClassData globalClassData);
  void removeGlobalClassData(String className);
  ClassData getGlobalClassData(String className);

  /**
   * INSTANCE
   */
  void storeInstance(InternalInstance instance);
  Set<InternalInstance> getInstances(ProcessDefinitionUUID processUUID);
  InternalInstance getInstance(ProcessInstanceUUID instanceUUID);
  InternalInstance removeInstance(ProcessInstanceUUID instanceUUID);
  InternalExecution getExecutionOnActivity(ProcessInstanceUUID instanceUUID,
      ActivityInstanceUUID activityInstanceUUID);

  /**
   * PROCESS
   */
  void storeProcess(InternalProcess process);

  InternalProcess getProcess(ProcessDefinitionUUID processUUID);
  InternalProcess removeProcess(ProcessDefinitionUUID processUUID);
  String getLastProcessVersion(String processId);
  void storeProcessVersion(String processId, String processVersion);
  void removeProcessVersion(String processId, String processVersion);

  /** returns a set of ProcessDependency of all packages depending on the given className. */
  Set<ProcessDefinitionUUID> getDependentProcesses(String className);

  /*
   * TIMERS
   */
  /** return all timers associated to the given instance */
  Set<Timer> getInstanceTimers(ProcessInstanceUUID instanceUUID);
  void removeTimer(Timer timer);
  InternalProcess findLatestProcessByName(String subProcessId);
  long getNextActivityInstanceNb(ProcessInstanceUUID instanceUUID,
      String activityName);
  long getNextProcessInstanceNb(ProcessDefinitionUUID processUUID);

  /*
   * Meta
   */
  void storeMetaData(String key, String value);
  String getMetaData(String key);
  void deleteMetaData(String key);
}
