/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.info;

import java.util.Map;

import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.Misc;


/**
 * @author Guillaume Porcher
 *
 */
// TODO javadoc
public class ActivityInstanceCurrentInfo {

	private final ProcessDefinitionUUID processUUID;
	private final ProcessInstanceUUID instanceUUID;
	private final ActivityInstanceUUID activityInstanceUUID;
	private final String activityName;
	private final String activityLabel;
	private final String activityDescription;
	private final String iterationId;
	private final String activityInstanceId;
	private final Map<String, Object> variables;
	private final boolean isAutomatic;

	public ActivityInstanceCurrentInfo(
			final ProcessDefinitionUUID processUUID,
			final ProcessInstanceUUID instanceUUID,
			final ActivityInstanceUUID activityInstanceUUID,
			final boolean isAutomatic,
			final String activityName,
			final String activityLabel,
			final String activityDescription,
			final String iterationId,
			final String activityInstanceId,
			final Map<String, Object> variables) {
		Misc.checkArgsNotNull(processUUID,
				instanceUUID,
				activityInstanceUUID,
				isAutomatic,
				activityName,
				iterationId,
				activityInstanceId);
		this.processUUID = processUUID;
		this.instanceUUID = instanceUUID;
		this.activityInstanceUUID = activityInstanceUUID;
		this.isAutomatic = isAutomatic;
		this.activityName = activityName;
		this.activityLabel = activityLabel;
		this.activityDescription = activityDescription;
		this.iterationId = iterationId;
		this.activityInstanceId = activityInstanceId;
		this.variables = variables;
	}

	public ProcessDefinitionUUID getProcessUUID() {
		return this.processUUID;
	}


	public ProcessInstanceUUID getInstanceUUID() {
		return this.instanceUUID;
	}


	public ActivityInstanceUUID getActivityInstanceUUID() {
		return this.activityInstanceUUID;
	}


	public boolean isAutomatic() {
		return this.isAutomatic;
	}


	public String getActivityName() {
		return this.activityName;
	}


	public String getIterationId() {
		return this.iterationId;
	}


	public String getActivityInstanceId() {
		return this.activityInstanceId;
	}


	public Map<String, Object> getVariables() {
		return this.variables;
	}

  public String getActivityLabel() {
    return this.activityLabel;
  }

  public String getActivityDescription() {
    return this.activityDescription;
  }

}
