/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.util;

import org.ow2.bonita.env.EnvironmentFactory;
import org.ow2.bonita.persistence.QuerierDbSession;
import org.ow2.bonita.persistence.RuntimeDbSession;

/**
 * This class holds the {@link EnvironmentFactory} singleton.
 *
 * The {@link EnvironmentFactory} returned by {@link #getEnvironmentFactory()} comes from the
 * parsing of the resource defined by the property
 * {@link #ENVIRONMENT_PROPERTY}. If this property has not been set, the
 * {@link #DEFAULT_ENVIRONMENT} resource is used and parsed.
 *
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public final class EnvTool extends FacadeEnvTool {

  private EnvTool() { }

  public static RuntimeDbSession getRuntimeDbSession(final String id) {
    return getEnvObject(RuntimeDbSession.class, id);
  }
  public static QuerierDbSession getQuerierDbSession(final String id) {
    return getEnvObject(QuerierDbSession.class, id);
  }

  @SuppressWarnings("unchecked")
  public static <T> T get(final Class<T> class1, final String key) {
    Misc.checkArgsNotNull(class1, key);
    final Object o = getEnv().get(key);
    String message = ExceptionManager.getInstance().getFullMessage(
  			"bis_ET_1", key);
    Misc.badStateIfNull(o, message);
    message = ExceptionManager.getInstance().getFullMessage(
  			"bis_ET_2", class1, o.getClass());
    Misc.badStateIfFalse(class1.isAssignableFrom(o.getClass()), message);
    return (T) o;
  }

}
